﻿using Microsoft.Extensions.Logging;
using ProjectBoat.CollectionGenericObjects;
using ProjectBoat.Drawnings;

namespace ProjectBoat
{
    public partial class FormBoatsCollection : Form
    {
        /// <summary>
        /// Хранилише коллекций
        /// </summary>
        private readonly StorageCollection<DrawningBoat> _storageCollection;

        /// <summary>
        /// Компания
        /// </summary>
        private AbstractCompany? _company = null;

        /// <summary>
        /// Логер
        /// </summary>
        private readonly ILogger _logger;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBoatsCollection(ILogger<FormBoatsCollection> logger)
        {
            InitializeComponent();
            _storageCollection = new();
            _logger = logger;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBoxSelectorCompany_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            panelCompanyTools.Enabled = false;
        }

        /// <summary>
        /// добавление крейсера
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {
            FormBoatConfing form = new();
            // TODO передать метод

            form.Show();
            form.AddEvent(SetBoat);
        }

        /// <summary>
        /// Добавление крейсера в коллекцию
        /// </summary>
        /// <param name="boat"></param>
        private void SetBoat(DrawningBoat? boat)
        {
            if (_company == null || boat == null)
            {
                return;
            }
            try
            {
                var res = _company + boat;
                MessageBox.Show("Объект добавлен");
                _logger.LogInformation($"Объект добавлен под индексом {res}");
                pictureBoxBoat.Image = _company.Show();
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Объект не добавлен: {ex.Message}", "Результат", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                _logger.LogError($"Ошибка: {ex.Message}", ex.Message);
            }
        }

        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosision.Text) || _company == null)
            {
                return;
            }
            if (MessageBox.Show("удалить объект?", "удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosision.Text);
            try
            {
                var res = _company - pos;
                MessageBox.Show("Объект удален");
                _logger.LogInformation($"Объект удален под индексом {pos}");
                pictureBoxBoat.Image = _company.Show();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Не удалось удалить объект",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError($"Ошибка: {ex.Message}", ex.Message);
            }
        }

        private void ButtonGetToTest_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            DrawningBoat? boat = null;
            int counter = 100;
            while (boat == null)
            {
                boat = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }
            if (boat == null)
            {
                return;
            }
            FormBoat form = new()
            {
                SetBoat = boat
            };
            form.ShowDialog();

        }

        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            pictureBoxBoat.Image = _company.Show();
        }

        /// <summary>
        /// Добавление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCollecctionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }

            try
            {
                _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
                _logger.LogInformation("Добавление коллекции");
                RerfreshListBoxItems();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError($"Ошибка: {ex.Message}", ex.Message);
            }
        }

        /// <summary>
        /// Удаленние коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCollectionDel_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }
            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            _logger.LogInformation("Коллекция удалена");
            RerfreshListBoxItems();
        }

        /// <summary>
        /// Обновление списка в listBoxCollection
        /// </summary>
        private void RerfreshListBoxItems()
        {
            listBoxCollection.Items.Clear();
            for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
            {
                string? colName = _storageCollection.Keys?[i];
                if (!string.IsNullOrEmpty(colName))
                {
                    listBoxCollection.Items.Add(colName);
                }
            }
        }

        /// <summary>
        /// Создание компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateCompany_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }

            ICollectionGenericObjects<DrawningBoat>? collection =
                _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
            if (collection == null)
            {
                MessageBox.Show("Коллекция не проинициализирована");
                return;
            }

            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new BoatDockingService(pictureBoxBoat.Width, pictureBoxBoat.Height, collection);
                    _logger.LogInformation("Компания создана");
                    break;
            }
            panelCompanyTools.Enabled = true;

        }

        /// <summary>
        /// Обработка нажатия "Сохранение"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storageCollection.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogError("Ошибка: {Message}", ex.Message);
                }
            }
        }

        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storageCollection.LoadData(openFileDialog.FileName);
                    RerfreshListBoxItems();
                    MessageBox.Show("Загрузка прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation("Загрузка из файла: {filename}", openFileDialog.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Загрузка не выполнена", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogError("Ошибка: {Message}", ex.Message);
                }
            }
        }
    }
}
